package de.draegerit.lambda;

import static org.junit.Assert.assertFalse;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;
import java.util.stream.Collectors;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TestSorted extends LambdaTestCase {

  private static Logger logger = LoggerFactory.getLogger("TestSorted.class");

  @Test
  public void testSortedBeforeJava8() {
    logger.info("sortierte Ausgabe");
    Collections.sort(names);
    for(String name:names) {
      logger.info(name);
    } 
  }
  
  @Test
  public void testSortedWithJava8() {
    logger.info("unsortierte Ausgabe");
    names.stream().forEach(name -> logger.info(name));

    List<String> sortedNames = names.stream().sorted().collect(Collectors.toList());
    assertFalse(sortedNames.isEmpty());
    logger.info("sortierte Ausgabe");
    sortedNames.stream().forEach(name -> logger.info(name));
  }
  
  @Test
  public void testSortedMapWithJava8() {
    final String formatPattern = "Key: [%s] \t Value: [%d]";
    
    //Map mit den Beispielwerten erzeugen.
    final Map<String,Integer> personen = new HashMap<>();
    names.stream().forEach(name->{
      personen.put(name, getRandomAge());
    });
    logger.info("unsortierte Ausgabe");
    personen.entrySet().stream().forEach(entry ->{
      logger.info(String.format(formatPattern, entry.getKey(),entry.getValue()));
    });
    
    //Sortieren der Map nach entry.getValue()
    Map<String,Integer> sortedbyValue = new LinkedHashMap<>();
    personen.entrySet().stream().sorted(Map.Entry.<String,Integer>comparingByValue()).forEachOrdered(person -> sortedbyValue.put(person.getKey(), person.getValue()));
    logger.info("sortierte Ausgabe - (sortedByValue)");
    sortedbyValue.entrySet().stream().forEach(entry ->{
      logger.info(String.format(formatPattern, entry.getKey(),entry.getValue()));
    });

    //Sortieren der Map nach entry.getKey()
    Map<String,Integer> sortedByKey = new LinkedHashMap<>();
    personen.entrySet().stream().sorted(Map.Entry.<String,Integer>comparingByKey()).forEachOrdered(person -> sortedByKey.put(person.getKey(), person.getValue()));
    logger.info("sortierte Ausgabe - (sortedByKey)");
    sortedByKey.entrySet().stream().forEach(entry ->{
      logger.info(String.format(formatPattern, entry.getKey(),entry.getValue()));
    });    
  }

  private Integer getRandomAge() {
    int min = 12;
    int max = 67;
    return ThreadLocalRandom.current().nextInt(min, max + 1);
  }
}
